#DEFINE TRI ANY !" \t\n\f\v"

TYPE Signale
  AS STRING Na, Ty, Pa
END TYPE
REDIM SHARED AS Signale SIGS(0)

'*
'*  reades the name of a signal definition
'*  liest den Namen einer Signaldefinition
'***********************************************************************
FUNCTION readSigName(BYVAL Fnr AS INTEGER) AS STRING
  DIM AS UBYTE z
  VAR s = "The """, p = 0, ls = LEN(s)
  DO
    IF EOF(Fnr) THEN RETURN ""
    GET #Fnr, , z
    p = IIF(s[p] <> z, 0, p + 1)
  LOOP UNTIL p >= ls
  VAR n = ""
  DO
    IF EOF(Fnr) THEN RETURN ""
    GET #Fnr, , z
    IF z <> ASC("""") THEN n += CHR(z) ELSE EXIT DO
  LOOP : s = " signal" : ls = LEN(s) : p = 0
  DO
    IF EOF(Fnr) THEN RETURN ""
    GET #Fnr, , z
    p = IIF(s[p] <> z, 0, p + 1)
  LOOP UNTIL p >= ls
  p = INSTR(n, "-")
  WHILE p
    MID(n, p, 1) = "_"
    p = INSTR(p + 1, n, "-")
  WEND : RETURN n
END FUNCTION

'*
'*  convertes a C-parameter definition into FB
'*  Konvertiert einen C-Parameter in einen FB-Typ
'***********************************************************************
FUNCTION convPara(BYREF T AS STRING) AS STRING
  VAR s = ""
  VAR p = 0
  FOR i AS INTEGER = LEN(T) - 1 TO 0 STEP -1
    SELECT CASE AS CONST T[i]
    CASE ASC(" ") : p = IIF(p, p, i)
    CASE ASC("*") : T[i] = ASC(" ") : p = IIF(p, p, i) : s &= " PTR"
    CASE ASC("-") : T[i] = ASC("_")
    END SELECT
  NEXT : RETURN TRIM(LEFT(T, p), TRI) & s & !"\t" & TRIM(MID(T, p + 2), TRI)
END FUNCTION

'*
'*  convertes a C-type into FB
'*  Konvertiert einen C-Variablentyp in einen FB-Typ
'***********************************************************************
FUNCTION readSigTyp(BYVAL Fnr AS INTEGER) AS STRING
  VAR a = "", n = "", s = "user_function",  p = 0, ls = LEN(s)
  DIM AS UBYTE z
  DO
    IF EOF(Fnr) THEN RETURN ""
    GET #Fnr, , z
    SELECT CASE AS CONST z
    CASE ASC("*") : z = ASC(" ") : a &= " PTR"
    CASE ASC("-") : z = ASC("_")
    END SELECT : IF z > 32 THEN n += CHR(z)
    p += IIF(s[p] <> z, 0, 1)
  LOOP UNTIL p >= ls : n = LEFT(n, LEN(n) - ls)
  IF n = "void" THEN RETURN ""
  RETURN n & a
END FUNCTION

'*
'*  reades a C-parameter definition
'*  Liest eine C-Parameterdefinition
'***********************************************************************
FUNCTION readSigPar(BYVAL Fnr AS INTEGER) AS STRING
  DIM AS STRING p, r
  DIM AS UBYTE z
  DO
    IF EOF(Fnr) THEN RETURN ""
    GET #Fnr, , z
  LOOP UNTIL z = ASC("(")
  WHILE NOT EOF(Fnr)
    GET #Fnr, , z
    SELECT CASE AS CONST z
    CASE ASC(")") : RETURN r & convPara(p)
    CASE ASC(",") : r &= convPara(p) & !"\t" : p = ""
    CASE ELSE : p += CHR(z)
    END SELECT
  WEND : RETURN ""
END FUNCTION

'*
'*  reades and convertes the Signals.def file (C into FB)
'*  Liest und konvertiert die Signals.def Datei (C into FB)
'***********************************************************************
FUNCTION readSignals(BYREF F AS STRING) AS STRING
  DIM AS INTEGER nr, fnr = FREEFILE
  DIM AS STRING na
  WITH SIGS(0)
    .Na = ""
    .Ty = ""
    .Pa = !"GtkWidget PTR\twidget\tgpointer\tuser_data" ' standard
  END WITH
  IF OPEN (F FOR BINARY ACCESS READ LOCK WRITE AS #fnr) THEN
    RETURN Tmsg(15) & NL & F & NL & STR(nr) & Tmsg(61)
  ELSE
    DO
      na = readSigName(fnr) : IF LEN(Na) = 0 THEN EXIT DO
      nr = UBOUND(SIGS) + 1
      REDIM PRESERVE AS Signale SIGS(nr)
      WITH SIGS(nr)
        .Na = na
        .Ty = readSigTyp(fnr)
        .Pa = readSigPar(fnr)
      END WITH
    LOOP UNTIL EOF(fnr) : CLOSE #fnr
  END IF : RETURN Tmsg(32) & NL & F & NL & STR(nr) & Tmsg(61)
END FUNCTION

'*
'*  finds the matching signal definition for Cl=class and N=signal name
'*  Findet die passende Signaldefinition
'***********************************************************************
FUNCTION findSig(BYREF W AS STRING, BYREF N AS STRING) AS STRING
  VAR t = ""
  FOR i AS INTEGER = UBOUND(SIGS) TO 1 STEP -1
    IF SIGS(i).Na = N THEN t += MKI(i)
  NEXT : IF LEN(t) = 4 THEN RETURN t
  IF LEN(t) = 0 THEN RETURN MKI(0)
  VAR c = LCASE(W), l = LEN(c)
  FOR i AS INTEGER = 1 TO LEN(t) STEP 4
    VAR j = CVI(MID(t, i, 4))
    IF LCASE(LEFT(SIGS(j).Pa, l)) = c THEN RETURN MKI(j)
  NEXT : RETURN MKI(0)
END FUNCTION

'*
'*  generates a FB parameter list from a stored parameter definition
'*  Erzeugt eine FB Parameterliste aus einer gespeicherten Definition
'***********************************************************************
FUNCTION genParList(BYREF P AS STRING, BYREF T AS STRING) AS STRING
  DIM AS UINTEGER a1 = 1, e1, a2, e2
  VAR r = ""
  DO
    e1 = INSTR(a1, P, !"\t")
    a2 = e1 + 1
    e2 = INSTR(a2, P, !"\t")
    r &= T & MID(P, a2, e2 - a2) & " AS " & MID(P, a1, e1 - a1)
    IF e2 THEN r &= "," ELSE EXIT DO
    a1 = e2 + 1
  LOOP : RETURN "(" & r & ")"
END FUNCTION

'*
'*  generates the FB source for the signal handler
'*  Erzeugt die FB-Quelltext fuer den Signal-Handler (Callback))
'***********************************************************************
FUNCTION genCB(BYREF T AS STRING) AS STRING
  DIM AS INTEGER nr = CVI(MID(t, 1, 4))
  DIM AS STRING r
  WITH SIGS(nr)
    IF LEN(.Ty) THEN r = "FUNCTION " ELSE r = "SUB "
    r &= MID(T, 5) & " CDECL ALIAS """ & MID(T, 5) & """ "
    r &= genParList(.Pa, !" _" & NL & "  BYVAL ")
    IF LEN(.Ty) THEN r &= " AS " & .Ty
    r &= " EXPORT"
    IF nr = 0 THEN r &= " ' Standard-Parameterliste"
    r &= NL & NL
    r &= "' place your source code here / eigenen Quelltext hier einfuegen" & NL
    r &= Tmsg(0) & NL & NL
    IF LEN(.Ty) THEN r &= "END FUNCTION" ELSE r &= "END SUB"
  END WITH : RETURN r & NL
END FUNCTION
